/* Copyright (c) 2001 - 2008 TOPP - www.openplans.org. All rights reserved.
 * This code is licensed under the GPL 2.0 license, available at the root
 * application directory.
 */
package org.geoserver.catalog;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.collections.map.LRUMap;
import org.geoserver.data.util.CoverageStoreUtils;
import org.geoserver.data.util.CoverageUtils;
import org.geoserver.feature.retype.RetypingDataStore;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.io.AbstractGridFormat;
import org.geotools.data.DataSourceException;
import org.geotools.data.DataStore;
import org.geotools.data.FeatureSource;
import org.geotools.factory.CommonFactoryFinder;
import org.geotools.factory.Hints;
import org.geotools.feature.AttributeTypeBuilder;
import org.geotools.feature.FeatureTypes;
import org.geotools.feature.simple.SimpleFeatureTypeBuilder;
import org.geotools.geometry.GeneralEnvelope;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.geotools.referencing.CRS;
import org.geotools.styling.SLDParser;
import org.geotools.styling.Style;
import org.geotools.styling.StyleFactory;
import org.geotools.util.logging.Logging;
import org.opengis.coverage.grid.GridCoverage;
import org.opengis.coverage.grid.GridCoverageReader;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.feature.simple.SimpleFeatureType;
import org.opengis.feature.type.AttributeDescriptor;
import org.opengis.feature.type.GeometryDescriptor;
import org.opengis.filter.Filter;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.TransformException;
import org.vfny.geoserver.global.GeoServerFeatureLocking;
import org.vfny.geoserver.global.GeoserverDataDirectory;
import org.vfny.geoserver.util.DataStoreUtils;

/**
 * Provides access to resources such as datastores, coverage readers, and 
 * feature types.
 * <p>
 * 
 * </p>
 * 
 * @author Justin Deoliveira, The Open Planning Project
 *
 */
public class ResourcePool {

    /**
     * Hint to specify if reprojection should occur while loading a 
     * resource.
     */
    public static Hints.Key REPROJECT = new Hints.Key( Boolean.class );
    
    /** logging */
    static Logger LOGGER = Logging.getLogger( "org.geoserver.catalog");

    HashMap<String, CoordinateReferenceSystem> crsCache;
    DataStoreCache dataStoreCache;
    FeatureTypeCache featureTypeCache;
    CoverageReaderCache coverageReaderCache;
    CoverageReaderCache hintCoverageReaderCache;
    HashMap<StyleInfo,Style> styleCache;
    
    public ResourcePool() {
        crsCache = new HashMap<String, CoordinateReferenceSystem>();
        dataStoreCache = new DataStoreCache();
        featureTypeCache = new FeatureTypeCache();
        coverageReaderCache = new CoverageReaderCache();
        hintCoverageReaderCache = new CoverageReaderCache();
        styleCache = new HashMap<StyleInfo, Style>();
        
    }
    
    /**
     * Returns a {@link CoordinateReferenceSystem} object based on its identifier
     * caching the result.
     * <p>
     * The <tt>srsName</tt> parameter should have one of the forms:
     * <ul>
     *   <li>EPSG:XXXX
     *   <li>http://www.opengis.net/gml/srs/epsg.xml#XXXX
     *   <li>urn:x-ogc:def:crs:EPSG:XXXX
     * </ul>
     * OR be something parsable by {@link CRS#decode(String)}.
     * </p>
     * @param srsName The coordinate reference system identifier.
     * 
     * @throws IOException In the event the srsName can not be parsed or leads 
     * to an exception in the underlying call to CRS.decode.
     */
    public CoordinateReferenceSystem getCRS( String srsName )
        throws IOException {
        
        CoordinateReferenceSystem crs = crsCache.get( srsName );
        if ( crs == null ) {
            synchronized (crsCache) {
                crs = crsCache.get( srsName );
                if ( crs == null ) {
                    try {
                        crs = CRS.decode( srsName );
                        crsCache.put( srsName, crs );
                    }
                    catch( Exception e) {
                        throw (IOException) new IOException().initCause(e);
                    }
                }
            }
        }
        
        return crs;
    }
    
    /**
     * Returns the underlying resource for a datastore, caching the result.
     * <p>
     * In the result of the resource not being in the cache {@link DataStoreInfo#getConnectionParameters()}
     * is used to connect to it.
     * </p>
     * @param info the data store metadata.
     * 
     * @throws IOException Any errors that occur connecting to the resource.
     */
    public DataStore getDataStore( DataStoreInfo info ) throws IOException {
        try {
            String name = info.getName();
            DataStore dataStore = (DataStore) dataStoreCache.get(name);
            if ( dataStore == null ) {
                synchronized (dataStoreCache) {
                    dataStore = (DataStore) dataStoreCache.get( name );
                    if ( dataStore == null ) {
                        //create data store
                        Map connectionParameters = info.getConnectionParameters();
                        
                        //call this methdo to execute the hack which recognizes 
                        // urls which are relative to the data directory
                        // TODO: find a better way to do this
                        connectionParameters = DataStoreUtils.getParams(connectionParameters,null);
                        dataStore = DataStoreUtils.getDataStore(connectionParameters);
                        
                        if ( dataStore == null ) {
                            throw new NullPointerException("Could not aquire datastore '" + info.getName() + "'");
                        }
                        
                        dataStoreCache.put( name, dataStore );
                    }
                } 
            }
            
            return dataStore;
        } 
        catch (Exception e) {
            throw (IOException) new IOException().initCause(e);
        }
    }
    
    /**
     * Clears the cached resource for a data store.
     * 
     * @param info The data store metadata.
     */
    public void clear( DataStoreInfo info ) {
        dataStoreCache.remove( info );
    }
    
    /**
     * Returns the underlying resource for a feature type, caching the result.
     * <p>
     * In the event that the resource is not in the cache the associated data store
     * resource is loaded, and the feature type resource obtained. During loading
     * the underlying feature type resource is "wrapped" to take into account 
     * feature type name aliasing and reprojection.
     * </p>
     * @param info The feature type metadata.
     * 
     * @throws IOException Any errors that occure while loading the resource.
     */
    public SimpleFeatureType getFeatureType( FeatureTypeInfo info ) throws IOException {
        
        SimpleFeatureType ft = (SimpleFeatureType) featureTypeCache.get( info );
        if ( ft == null ) {
            synchronized ( featureTypeCache ) {
                ft = (SimpleFeatureType) featureTypeCache.get( info );
                if ( ft == null ) {
                    
                    //grab the underlying feature type
                    DataStore dataStore = getDataStore(info.getStore());
                    ft = (SimpleFeatureType) dataStore.getSchema( info.getNativeName() );
                    
                    //create the feature type so it lines up with the "declared" schema
                    SimpleFeatureTypeBuilder tb = new SimpleFeatureTypeBuilder();
                    tb.setName( info.getName() );
                    tb.setNamespaceURI( info.getNamespace().getURI() );

                    for ( AttributeDescriptor ad : ft.getAttributeDescriptors() ) {
                    //for ( AttributeTypeInfo att : info.getAttributes() ) {
                    //    String attName = att.getName();
                    //    
                    //    //load the actual underlying attribute type
                    //    AttributeDescriptor ad = ft.getAttribute( attName );
                    //    if ( ad == null ) {
                    //        throw new IOException("the SimpleFeatureType " + info.getPrefixedName()
                    //                + " does not contains the configured attribute " + attName
                    //                + ". Check your schema configuration");
                    //    }

                        // force the user specified CRS if the data has no CRS, or reproject it 
                        // if necessary
                        if ( ad instanceof GeometryDescriptor ) {
                            GeometryDescriptor old = (GeometryDescriptor) ad;
                            try {
                                //if old has no crs, change the projection handlign policy
                                // to be the declared
                                boolean rebuild = false;
                                
                                if ( old.getCoordinateReferenceSystem() == null ) {
                                    //(JD) TODO: this is kind of wierd... we should at least
                                    // log something here, and this is not thread safe!!
                                    info.setProjectionPolicy(ProjectionPolicy.FORCE_DECLARED);
                                    rebuild = true;
                                }
                                else {
                                    ProjectionPolicy projPolicy = info.getProjectionPolicy();
                                    if ( projPolicy == ProjectionPolicy.REPROJECT_TO_DECLARED || 
                                        projPolicy == ProjectionPolicy.FORCE_DECLARED ) {
                                        rebuild = true;
                                    }
                                }
                                
                                if ( rebuild ) {
                                    //rebuild with proper crs
                                    AttributeTypeBuilder b = new AttributeTypeBuilder();
                                    b.init(old);
                                    b.setCRS( getCRS(info.getSRS()) );
                                    ad = b.buildDescriptor(old.getLocalName());
                                }
                            }
                            catch( Exception e ) {
                                //log exception
                            }
                        
                        }
                        tb.add( ad );
                    }

                    ft = tb.buildFeatureType();
                    featureTypeCache.put( info, ft ); 
                }
            }
        }
        
        return ft;
    }

    /**
     * Clears a feature type resource from the cache.
     * 
     * @param info The feature type metadata.
     */
    public void clear( FeatureTypeInfo info ) {
        featureTypeCache.remove( info );
    }
    
    /**
     * Loads the feature source for a feature type.
     * <p>
     * The <tt>hints</tt> parameter is used to control how the feature source is 
     * loaded. An example is using the {@link #REPROJECT} hint to control if the 
     * resulting feature source is reprojected or not.
     * </p>
     * @param info The feature type info.
     * @param hints Any hints to take into account while loading the feature source, 
     *  may be <code>null</code>.
     * 
     * @throws IOException Any errors that occur while loading the feature source.
     */
    public FeatureSource getFeatureSource( FeatureTypeInfo info, Hints hints ) throws IOException {
        DataStore dataStore = getDataStore(info.getStore());
        FeatureSource<SimpleFeatureType, SimpleFeature> fs;
        
        //
        // aliasing
        //
        if ( !info.getName().equals( info.getNativeName() ) ) {
            final String typeName = info.getNativeName();
            final String alias = info.getName();
            
            RetypingDataStore retyper = new RetypingDataStore(dataStore) {
            
                @Override
                protected String transformFeatureTypeName(String originalName) {
                    if(!typeName.equals(originalName))
                        return originalName;
                    return alias;
                }
            
            };
            fs = retyper.getFeatureSource(alias);
        }
        else {
            //normal case
            fs = dataStore.getFeatureSource(info.getName());   
        }

        //
        // reprojection
        //
        Boolean reproject = Boolean.TRUE;
        if ( hints != null ) {
            if ( hints.get( REPROJECT ) != null ) {
                reproject = (Boolean) hints.get( REPROJECT );
            }
        }
        
        //get the reprojection policy
        ProjectionPolicy ppolicy = info.getProjectionPolicy();
        
        //if projection policy says to reproject, but calling code specified hint 
        // not to, respect hint
        if ( ppolicy == ProjectionPolicy.REPROJECT_TO_DECLARED && !reproject) {
            ppolicy = ProjectionPolicy.NONE;
        }
        
        List<AttributeTypeInfo> attributes = info.getAttributes();
        if (attributes == null || attributes.isEmpty()) { 
            return fs;
        } 
        else {
            CoordinateReferenceSystem resultCRS = null;
            GeometryDescriptor gd = fs.getSchema().getGeometryDescriptor();
            CoordinateReferenceSystem nativeCRS = gd != null ? gd.getCoordinateReferenceSystem() : null;
            
            if (ppolicy == ProjectionPolicy.NONE && nativeCRS != null) {
                resultCRS = nativeCRS;
            }
            else {
                resultCRS = getCRS(info.getSRS());
            }

            // make sure we create the appropriate schema, with the right crs
            SimpleFeatureType schema = getFeatureType(info);
            try {
                if (!CRS.equalsIgnoreMetadata(resultCRS, schema.getCoordinateReferenceSystem()))
                    schema = FeatureTypes.transform(schema, resultCRS);
            } catch (Exception e) {
                throw new DataSourceException(
                        "Problem forcing CRS onto feature type", e);
            }

            //
            //versioning
            //
            try {
                //only support versioning if on classpath
                Class clazz = Class.forName("org.geotools.data.VersioningFeatureSource");
                if ( clazz.isAssignableFrom( fs.getClass() ) ) {
                    //class implements versioning, reflectively create the 
                    // versioning wrapper
                    try {
                    Class wrapper = 
                        Class.forName("org.vfny.geoserver.global.GeoServerVersioningFeatureSource");
                    Method m = wrapper.getMethod( "create", clazz, 
                        SimpleFeatureType.class, Filter.class, CoordinateReferenceSystem.class, int.class );
                    return (FeatureSource) m.invoke(null, fs, schema, info.getFilter(), 
                        resultCRS, info.getProjectionPolicy().getCode());
                    }
                    catch( Exception e ) {
                        throw new DataSourceException(
                                "Creation of a versioning wrapper failed", e);
                    }
                }
            }
            catch( ClassCastException e ) {
                //fall through
            } catch (ClassNotFoundException e) {
                //fall through
            }
            
            //return a normal 
            return GeoServerFeatureLocking.create(fs, schema,
                    info.getFilter(), resultCRS, info.getProjectionPolicy().getCode());
        }
    }
    
    /**
     * Returns a coverage reader, caching the result.
     *  
     * @param info The coverage metadata.
     * @param hints Hints to use when loading the coverage, may be <code>null</code>.
     * 
     * @throws IOException Any errors that occur loading the reader.
     */
    public GridCoverageReader getGridCoverageReader( CoverageStoreInfo info, Hints hints ) 
        throws IOException {
        
        GridCoverageReader reader = null;
        if ( hints != null ) {
            reader = (GridCoverageReader) hintCoverageReaderCache.get( info );    
        }
        else {
            reader = (GridCoverageReader) coverageReaderCache.get( info );
        }
        
        if (reader != null) {
            return reader;
        }

        
        synchronized ( hints != null ? hintCoverageReaderCache : coverageReaderCache ) {
            /////////////////////////////////////////////////////////
            //
            // Getting coverage reader using the format and the real path.
            //
            // /////////////////////////////////////////////////////////
            final File obj = GeoserverDataDirectory.findDataFile(info.getURL());

            // XXX CACHING READERS HERE
            reader = (info.getFormat()).getReader(obj,hints);
            (hints != null ? hintCoverageReaderCache : coverageReaderCache ).put(info, reader); 
        }
        
        return reader;
            
    }
    
    /**
     * Clears any cached readers for the coverage.
     */
    public void clear(CoverageStoreInfo info) {
        coverageReaderCache.remove(info);
    }
 
    /**
     * Loads a grid coverage.
     * <p>
     * 
     * </p>
     * 
     * @param info The grid coverage metadata.
     * @param envelope The section of the coverage to load. 
     * @param hints Hints to use while loading the coverage.
     * 
     * @throws IOException Any errors that occur loading the coverage.
     */
    public GridCoverage getGridCoverage( CoverageInfo info, ReferencedEnvelope env, /*Rectangle dim,*/ Hints hints) 
        throws IOException {
        
        ReferencedEnvelope coverageBounds;
        try {
            coverageBounds = info.getBoundingBox();
        } 
        catch (Exception e) {
            throw (IOException) new IOException( "unable to calculate coverage bounds")
                .initCause( e );
        }
        
        GeneralEnvelope envelope = null;
        if (env == null) {
            envelope = new GeneralEnvelope( coverageBounds );
        }
        else {
            envelope = new GeneralEnvelope( env );
        }
    
        // /////////////////////////////////////////////////////////
        //
        // Do we need to proceed?
        // I need to check the requested envelope in order to see if the
        // coverage we ask intersect it otherwise it is pointless to load it
        // since its reader might return null;
        // /////////////////////////////////////////////////////////
        final CoordinateReferenceSystem sourceCRS = envelope.getCoordinateReferenceSystem();
        CoordinateReferenceSystem destCRS;
        try {
            destCRS = info.getCRS();
        } 
        catch (Exception e) {
            throw (IOException) new IOException( "unable to determine coverage crs").initCause(e);
        }
        
        if (!CRS.equalsIgnoreMetadata(sourceCRS, destCRS)) {
            try {
	            // get a math transform
	            final MathTransform transform = CRS.findMathTransform(sourceCRS, destCRS,true);
	        
	            // transform the envelope
	            if (!transform.isIdentity()) {
	
	                    envelope = CRS.transform(transform, envelope);
	            } 
            }
            catch (TransformException e) {
                throw (IOException) new IOException( "error occured transforming envelope").initCause( e );
            
            } catch (FactoryException e) {
            	throw (IOException) new IOException( "error occured transforming envelope").initCause( e );
			}
            
        }
        
        // just do the intersection since
        envelope.intersect(coverageBounds);
        
        if (envelope.isEmpty()) {
            return null;
        }
        
        envelope.setCoordinateReferenceSystem(destCRS);
        
        // /////////////////////////////////////////////////////////
        //
        // get a reader
        //
        // /////////////////////////////////////////////////////////
        final GridCoverageReader reader = getGridCoverageReader(info.getStore(),hints);
        
        if (reader == null) {
            return null;
        }
        
        // /////////////////////////////////////////////////////////
        //
        // Reading the coverage
        //
        // /////////////////////////////////////////////////////////
        
        GridCoverage gc  = reader.read(CoverageUtils.getParameters(
                    reader.getFormat().getReadParameters(), info.getParameters()));
        
        if ((gc == null) || !(gc instanceof GridCoverage2D)) {
            throw new IOException("The requested coverage could not be found.");
        }
        
        return gc;
    }

    /**
     * Returns the format for a coverage.
     * <p>
     * The format is inferred from {@link CoverageStoreInfo#getType()}
     * </p>
     * @param info The coverage metadata.
     * 
     * @return The format, or null.
     */
    public AbstractGridFormat getGridCoverageFormat( CoverageStoreInfo info ) {
        final int length = CoverageStoreUtils.formats.length;

        for (int i = 0; i < length; i++) {
            if (CoverageStoreUtils.formats[i].getName().equals(info.getType())) {
                return (AbstractGridFormat) CoverageStoreUtils.formats[i];
            }
        }

        return null;
    }
    
    /**
     * Returns a style resource, caching the result.
     * <p>
     * The resource is loaded by parsing {@link StyleInfo#getFilename()} as an 
     * SLD.
     * </p>
     * @param info The style metadata.
     * 
     * @throws IOException Any parsing errors.
     */
    public Style getStyle( StyleInfo info ) throws IOException {
        Style style = styleCache.get( info );
        if ( style == null ) {
            synchronized (styleCache) {
                style = styleCache.get( info );
                if ( style == null ) {
                    String filename = info.getFilename();
                    
                    StyleFactory styleFactory = CommonFactoryFinder.getStyleFactory(null);
                    File styleFile = GeoserverDataDirectory.findStyleFile(filename);
                    SLDParser stylereader;
                  
                    try {
                        stylereader = new SLDParser(styleFactory, styleFile);
                    } catch (FileNotFoundException e) {
                        throw (IOException) new IOException().initCause(e);
                    }
                    
                    style = stylereader.readXML()[0];
                    //set the name of the style to be the name of hte style metadata
                    // remove this when wms works off style info
                    style.setName( info.getName() );
                    styleCache.put( info, style );
                }
            }
        }
        
        return style;
    }
    
    /**
     * Clears a style resource from the cache.
     * 
     * @param info The style metadata.
     */
    public void clear(StyleInfo info) {
        styleCache.remove( info );
    }
    /**
     * Disposes all cached resources.
     *
     */
    public void dispose() {
        crsCache.clear();
        dataStoreCache.clear();
        featureTypeCache.clear();
        coverageReaderCache.clear();
        hintCoverageReaderCache.clear();
    }
    
    static class FeatureTypeCache extends LRUMap {
        
        protected boolean removeLRU(LinkEntry entry) {
            FeatureTypeInfo info = (FeatureTypeInfo) entry.getKey();
            LOGGER.info( "Disposing feature type '" + info.getName() + "'");
            
            return super.removeLRU(entry);
        }
    }
    
    static class DataStoreCache extends LRUMap {
        protected boolean removeLRU(LinkEntry entry) {
            String name = (String) entry.getKey();
            dispose(name,(DataStore) entry.getValue());
            
            return super.removeLRU(entry);
        }
        
        void dispose(String name, DataStore dataStore) {
            LOGGER.info( "Disposing datastore '" + name + "'" );
            
            try {
                dataStore.dispose();
            }
            catch( Exception e ) {
                LOGGER.warning( "Error occured disposing datastore '" + name + "'");
                LOGGER.log(Level.FINE, "", e );
            }
            
        }
        
        protected void destroyEntry(HashEntry entry) {
            dispose( (String) entry.getKey(), (DataStore) entry.getValue() );
            super.destroyEntry(entry);
        }
        
        public void clear() {
            for ( Iterator e = entrySet().iterator(); e.hasNext(); ) {
                Map.Entry<String,DataStore> entry = 
                    (Entry<String, DataStore>) e.next();
                dispose( entry.getKey(), entry.getValue() );
            }
            super.clear();
        }
    }
    
    static class CoverageReaderCache extends LRUMap {
        protected boolean removeLRU(LinkEntry entry) {
            CoverageStoreInfo info = (CoverageStoreInfo) entry.getKey();
            dispose( info, (GridCoverageReader) entry.getValue() );
            return super.removeLRU(entry);
        }
        
        void dispose( CoverageStoreInfo info, GridCoverageReader reader ) {
            LOGGER.info( "Disposing grid coverage reader '" + info.getName() + "'");
            try {
                reader.dispose();
            }
            catch( Exception e ) {
                LOGGER.warning( "Error occured disposing coverage reader '" + info.getName() + "'");
                LOGGER.log(Level.FINE, "", e );
            }
        }
        
        protected void destroyEntry(HashEntry entry) {
            dispose( (CoverageStoreInfo) entry.getKey(), (GridCoverageReader) entry.getValue() );
            super.destroyEntry(entry);
        }
        
        public void clear() {
            for ( Iterator e = entrySet().iterator(); e.hasNext(); ) {
                Map.Entry<CoverageStoreInfo,GridCoverageReader> entry = 
                    (Entry<CoverageStoreInfo, GridCoverageReader>) e.next();
                dispose( entry.getKey(), entry.getValue() );
            }
            super.clear();
        }
    }
    
}
